<?php

/*
This class contains all functions needed to display the articles and their categories.

Init the class in template_globals.php as a global object.
*/

// Init the class in get_template_globals.php as a global object.
class ArticlesManager {


    private $template;
    private $articles;
    private $articles_url;
    private $max_articles_per_page;
    private $logging;
    private $sef_urls_enabled = false;

    public function __construct($template, $logging) {

        $this->template = $template;
        $this->articles = $template['articles'];
        $this->articles_url = $template['articles']['articlesCatalogURL'];
        $this->logging = $logging;
        $this->max_articles_per_page = 20;

        if ( strpos($this->articles_url, 'articles.php') == false ) {
            $this->sef_urls_enabled = true;
        }

        $this->log('Initializing ArticlesManager');
        $this->log($this->articles_url);

    }


    private function log($message) {

        if ($this->logging) {
            global $debug_message_list;
            array_push($debug_message_list, $message);
        }

    }


    // Returns the category's URL. If SEF URLs are not enabled it generates a URL based on the category's ID.
    public function category_url($sef_url=null, $category_id=null) {

        if ( $this->sef_urls_enabled === true ) {

            return $sef_url;

        } else {

            // Generate a URL from the catergory id.
            if ( ! empty($category_id) ) {
                if ( empty($this->template['language']) ) {
                    $lang = 'l=' . $this->template['language'];
                    return $this->articles_url . '?articleCategoryID=' . $category_id;
                }
                return $this->articles_url . '?' . $lang . '&articleCategoryID=' . $category_id;
            } else {
                $this->log('category_url was called without any valid parameters');
            }

        }

    }


    // Return the previous url.
    // Get the URL from HTTP_REFERER and return it unless it does not belong to our website or it doesn't exist.
    // In that case return the articles' or home URL.
    public function article_back_url() {

        $prev_url = $_SERVER['HTTP_REFERER'];
        if ( strpos($prev_url, $_SERVER['HTTP_HOST']) == false ) {
            if ( ! empty($this->template['article']['catalogURL']) ) {
                $prev_url = $this->template['article']['catalogURL'];
            } else {
                $prev_url = $_SERVER['HTTP_HOST'];
            }
        }
        return $prev_url;

    }


    // This function applies fake pagination on the articles that are inside a category. Hopefully, this will be
    // replaced in the future by proper pagination done in the database query layer.
    private function show_paged_articles() {

        $total_pages = $this->articles['totalPages'];
        $current_page = $this->articles['pageNumber'];

        $start_index = $this->max_articles_per_page * ($current_page - 1);
        $end_index = $this->max_articles_per_page * $current_page;

        $articles_list = array_slice($this->articles['categories']['articles'], $start_index, $end_index);

        $this->log('Showing paged articles.');

        return $articles_list;

    }


    // Filter articles by category if there is a selected category, else return all articles.
    public function articles_list() {

        if ( ! empty($this->articles['categories']['selectedCategoryName']) ) {

            if ( ! empty($this->articles['totalPages']) ) {
                return $this->show_paged_articles();
            } else {
                return $this->articles['categories']['articles'];
            }

        } else {
            return $this->articles['allArticles'];
        }

    }


    // Generate a breadctumb of the category tree we are in.
    public function breadcrumb() {

        if ( ! empty($this->template['articles']['categories']['selectedCategoryName']) ) {
            echo '<nav aria-label="breadcrumb">';
                echo '<ol class="breadcrumb">';
                    echo '<li class="breadcrumb-title">' . $this->template['translate']['category'] . ': </li>';
                    if ( empty($this->template['articles']['categories']['selectedCategoryParentName']) ) {
                        echo '<li class="breadcrumb-item active" aria-current="page">' . $this->template['articles']['categories']['selectedCategoryName'] . '</li>';
                    } else {
                        echo '<li class="breadcrumb-item"><a href="' . category_url($this->template['articles']['categories']['selectedCategoryParentSEFURL'], $this->template['articles']['categories']['selectedCategoryParentID']) . '">' . $this->template['articles']['categories']['selectedCategoryParentName'] . '</a></li>';
                        echo '<li class="breadcrumb-item active" aria-current="page">' . $this->template['articles']['categories']['selectedCategoryName'] . '</li>';
                    }
                echo '</ol>';
            echo '</nav>';
        }

    }


    // Show pagination controls if available or if we exceed the max_articles_per_page.
    // This method was written because the $template['articles']['totalPages'] does not get updated when we are in a
    // category, thus showing the pagination constrols even if the category has only a few articles.
    public function pagination($category_articles) {

        $total_category_articles = count($category_articles['categories']['articles']) / 20;

        $show_pag = false;

        if ( empty($this->template['articles']['categories']['selectedCategoryName']) ) {
            if (  $this->template['articles']['totalPages'] > 1 ) {
                $this->log('selectedCategoryName is empty and totalPages is greater than 1.');
                $show_pag = true;
            }
        } else {
            if ( count($this->template['articles']['categories']['articles']) > $this->max_articles_per_page ) {
                $this->log('The articles in this category are more than ' . $this->max_articles_per_page);
                $show_pag = true;
            }
        }

        if ( $show_pag === true ) {
            $this->log('Showing pagination.');
            echo '<div id="articles-pagination" class="col-md-12">';
            uiPrintArticlesPaging($this->template['articles']['pageNumber'], intval($total_category_articles) + 1, $this->template['translate']);
            echo '</div>';
            return true;
        } else {
            return false;
        }

    }


    // Generate a menu and submenus of the articles' categories.
    public function create_articles_menu($main_menu) {

        $this->log('Creating articles menu');
        $categories = $this->template['articles']['categories']['categoryTree'];
        $categories_submenu = $this->create_categories_submenu($categories);

        // Find the articles menu and turn it into a dropdown with the categories submenu.
        if ( ! empty($this->articles_url) ) {
            foreach ( $main_menu['items'] as $menu_index => $menu_content ) {
                if ( $menu_content['url'] == $this->articles_url ) {
                    $menu_content['itemClass'] = 'dropdown';
                    $menu_content['dropdown'] = true;
                    $menu_content['subMenu'] = $categories_submenu;
                    $main_menu['items'][$menu_index] = $menu_content;
                    break;
                }
            }
        }

        return $main_menu;

    }


    // Iterate over the categories and their children recursively and turn them into menu items.
    // Returns the $category_tree as a submenu.
    private function create_categories_submenu($category_tree) {

        foreach ( $category_tree as $cat_index => $cat_content ) {
            $this->log('Creating menu item for: ' . $cat_content['title'] . ' id: ' . $cat_content['id']);

            $new_cat_item = array(
                'title' => $cat_content['title'],
                'url' => $this->category_url($cat_content['sefURL'], $cat_content['id']),
                'dropdown' => false,
                'itemClass' => '',
                'class' => '',
                'subMenu' => array(),
                'target' => ''
            );

            // If this category has children, then turn this menu item into a dropdown and append a submenu of its
            // children.
            if ( ! empty($cat_content['children']) ) {
                $new_cat_item['itemClass'] = 'dropdown';
                $new_cat_item['dropdown'] = true;
                $new_cat_item['subMenu'] = $this->create_categories_submenu($cat_content['children']);
            }

            $category_tree[$cat_index] = $new_cat_item;

        }

        return $category_tree;

    }

}


?>